// usage: java HandleErrors names-ages.txt
// 
// Read records of the format
//   firstname   lastname   age
//   firstname   lastname   age
//   firstname   lastname   age
// and print the results.
// 
// Read records catching errors of malformed lines; recover gracefully
// from them using exceptions

// This version is monolithic - no functions
import java.io.*;
import java.util.*;
public class HandleErrors{
  public static final int MAX_ERRORS = 5;

  public static void main(String args[]){
    String filename = args[0];
    String first, last;
    int age;
    int lineNum=0, errorCount=0;
    Scanner f = null;
    try {
      f = new Scanner(new File(filename));
      for(lineNum=1, errorCount=0; f.hasNext(); lineNum++){
	try {			// Handle line errors gracefully
	  String line = f.nextLine();
	  Scanner oneLine = new Scanner(line);
	  first = oneLine.next();
	  last = oneLine.next();
	  age = oneLine.nextInt();
	  System.out.printf("Successfully read %s %s %d\n",
			    first, last, age);

	}
	catch(Exception e){
	  System.err.printf("Line %d malformed, skipping\n", lineNum);
	  errorCount++;
	  if(errorCount >= MAX_ERRORS){
	    throw new Exception("To many errors");
	  }
	}
      }
      System.out.printf("Read %d lines, %d errors\n",
			lineNum-1, errorCount);

    }
    catch(FileNotFoundException e){
      System.err.println("Error: "+e.getMessage());
    }
    catch(Exception e){
      System.err.println("General error: "+e.toString());
    }
    finally{			// Run whether or not errors occur
      if(f != null){
	System.out.println("Closing the file");
	f.close();
      }
    }
  }
}
