// usage: java HandleErrors names-ages.txt
// 
// Read records of the format
//   firstname   lastname   age
//   firstname   lastname   age
//   firstname   lastname   age
// and print the results.
// 
// Read records catching errors of malformed lines; recover gracefully
// from them using exceptions
// 
// This version breaks reading into functions for lines and files
import java.io.*;
import java.util.*;
public class HandleErrors2{
  public static final int MAX_ERRORS = 5;

  public static void main(String args[]){
    String filename = args[0];
    try {
      readFile(filename);
    }
    catch(FileNotFoundException e){
      System.err.println("Error: "+e.getMessage());
    }
  }

  // Read a single line and print its contents. May throw an exception
  // if the line is malformed (NoSuchElement or format conversion
  // problem)
  public static void readLine(Scanner f){
    String line = f.nextLine();
    Scanner oneLine = new Scanner(line);
    String first = oneLine.next();
    String last = oneLine.next();
    int age = oneLine.nextInt();
    System.out.printf("Successfully read %s %s %d\n",
		      first, last, age);
  }

  // Read the whole file. Handle line errors. Bail out if the number
  // of errors gets too high.
  public static void readFile(String fname) throws FileNotFoundException{
    int lineNum=0, errorCount=0;
    Scanner f = new Scanner(new File(fname));
    try{
      for(lineNum=1, errorCount=0; f.hasNext(); lineNum++){
	try {			// Handle line errors gracefully
	  readLine(f);
	}
	catch(Exception e){
	  System.err.printf("Line %d malformed, skipping\n", lineNum);
	  errorCount++;
	  if(errorCount >= MAX_ERRORS){
	    throw new Exception("To many errors");
	  }
	}
      }
      System.out.printf("Read %d lines, %d errors\n", lineNum-1, errorCount);
    }
    catch(Exception e){
      System.err.println("General error: "+e.toString());
    }
    finally{			// Run whether or not errors occur
      if(f != null){
	System.out.println("Closing the file");
	f.close();
      }
    }
  }

}
